/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2016, 2018 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const commandLineArgs = require('command-line-args');
var workManager = require('./modules/workManager');
var zipper = require('./modules/zipper');
// declare winston as global
global.winston = require('winston');
// Initialize configuration values
global.config = require(__dirname + '/modules/config');

var user_config = null;
winston.info("User config path "+__dirname + '/../user_config');
if(config.devMode){
   user_config = require(__dirname + '/../../Bluemix_Design_Room/user_config');
   winston.info("DEVMODE IS ON");
}
else{
   user_config = require(__dirname + '/../user_config');
}

// Merge user configuration values
Object.getOwnPropertyNames(user_config).forEach(
	function (propName, idx, array) {
		config[propName] = user_config[propName];
	}
);

global.requester = require('./modules/requester');

winston.add(winston.transports.File, 
				{
					filename: 'logs/drlimpexp' + Date.now() + '.log',
					maxFiles: 1,
					prettyPrint: true
				});



const optionDefinitions = [
	{ name: 'tag', alias: 't', type: String },
	{ name: 'export', alias: 'e', type: Boolean },
	{ name: 'exportUsers', alias : 'x',type: Boolean },
	{ name: 'import', alias: 'i', type: Boolean },
	{ name: 'importUsers', alias : 'u',type: Boolean },
	{ name: 'deleteAll', type: Boolean },
	{ name: 'token', alias : 'k',type: String },
	{ name: 'username', alias : 'n',type: String },
	{ name: 'password', alias : 'p',type: String },
	{ name: 'whiteboards', alias: 'w', type : String, multiple:true}
];


const options = commandLineArgs(optionDefinitions);

winston.info('Design Room Live! Export/Import Started');
winston.info('Child process argument options', options);
winston.info("Child Process merged options " + config);

workManager.workStarted();

var expImpPath = config.exportImportPath + options.tag;
winston.info('Child Process: Path Import Export', expImpPath);

if(options.export || options.exportUsers){ //safeguard against combinations of import and export options
	executeExportOperations();
}
else if(options.import || options.importUsers){
	
	if(options.token){//token reveived
		//TO DO: We could check for validity of token just in case but not crucial
		requester.setToken(options.token);
		executeImportOperations();
	}
	else{//username and password 
		if(!options.username||!options.password){
			winston.error("Credentials are missing");
			process.exit(1);
		}
		//proceed
		requester.authenticate(options.username, options.password, function(res){
			if(!res) process.exit(1);
			winston.log("Token generated and expires "+JSON.parse(res).expires);
			executeImportOperations();

		})
	}
}

workManager.workEnded();

setInterval(function(){
	if(workManager.getWorkCount()==0){//no more work to process
		if(options.export){//only zipping after successful export
			zipper.init(expImpPath);
			zipper.doZip(function(err){
				if(err)
					process.exit(1);
				process.exit(0);
			});
		}
		else{
			process.exit(0);			
		}
	}
},1000)

function executeImportOperations(){
	if(!options.tag){//command line checks
		winston.error("Tag is missing")
		process.exit(1);
	}

	if (options.import) {
		var importer = require('./modules/importer')();
		//Delete all existing whiteboards as part of import process
		var deleteAll= (options.deleteAll)?true:false;
		importer.doImport(expImpPath, deleteAll);
	}

	if (options.importUsers) {
		var importer = require('./modules/UsersImporter')();
		importer.doImport(expImpPath);
	}

}

function executeExportOperations(){
	if(!options.tag){//command line checks
		winston.error("Tag is missing")
		process.exit(1);
	}

	if (options.export) {
		var exporter = require('./modules/exporter')();
		//Exporting explicitely passed list of whiteboards
		if(options.whiteboards)
			exporter.doExportPartial(expImpPath, options.whiteboards);
		else//export all whiteboards in room
			exporter.doExportAll(expImpPath);
	}

	if (options.exportUsers) {
		var exporter = require('./modules/usersExporter')();
		exporter.doExport(expImpPath);
	}
}