/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2018 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

	var workManager = require('../modules/workManager');
	var request = require('request');
	var Requester = {};
	var port=config.sslPort || 443;
	var host=process.env.VCAP_APP_HOST || "localhost";
	var baseURL = "https://"+host+":"+port;
	
	Requester.authenticate = function (username, password, done) {
		//set options
		var reqOptions = { method: 'POST',
		url: baseURL+'/api/authenticate',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
		   'cache-control': 'no-cache' 
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  },
		form: { username: username, password: password } };
		
		//send request
	  request(reqOptions, function (error, response, body) {
			//err
			if (error) {
				winston.error("There seems to be a server issue");
				process.exit(1);
			}

			//proceed
			winston.info("Token is generated");
			Requester.setToken(JSON.parse(body).token);
			done(body);
	  });
	
	}
	Requester.setToken = function(token){
		Requester["token"] = token;
		winston.info("Token is set")
	}
	Requester.getToken = function(token){
		return Requester["token"]?Requester["token"]:null;
	}
	Requester.addCells = function (room_id, graph_id, cells, done ){
		//set options
		var reqOptions = { method: 'POST',
		url: baseURL+'/api/addCells',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken()
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  },
			form: { room_id: room_id, graph_id: graph_id, cells : cells } 
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			winston.info("whiteboard:"+graph_id+ " in "+room_id+" was cleared successfully");
			done(null);
	  });

	}

	Requester.clearWhiteboard = function (room_id, graph_id, done ){
		//set options
		var reqOptions = { method: 'DELETE',
		url: baseURL+'/api/clearWhiteboard',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken()
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  },
			form: { room_id: room_id, graph_id: graph_id } 
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			winston.info("whiteboard:"+graph_id+ " in "+room_id+" was cleared successfully");
			done(null);
	  });

	}
	Requester.resolveAllModelChanges = function (done){
		//set options
		var reqOptions = { method: 'PUT',
		url: baseURL+'/modelElements/resolve',
		headers: 
		 { 'content-type': 'application/x-www-form-urlencoded',
			 'cache-control': 'no-cache',
			 'x-access-token': Requester.getToken()
		 },
		 agentOptions: {
			rejectUnauthorized: false
		  }
		};
		
		//place request
	  request(reqOptions, function (error, response, body) {
			if (error) return done(error);

			//send response
			winston.info("Model changes propagations completed successfully");
			done(null);
	  });

	}
	module.exports = Requester;
