/************************************************************************/
/* Licensed Materials – Property of  HCL                                */
/* (c) Copyright HCL Technologies Ltd. 2016, 2017 All rights reserved.  */
/* Trademark of HCL Technologies Limited                                */
/************************************************************************/

const fs = require('fs-extra');
const async = require('async');
var workManager = require('../modules/workManager');

module.exports = function () {
	var Importer = {};

	Importer.doImport = function (expImpPath) {
		var config = require('../modules/config');
		var Users = require('../models/user');

		var usersFile = expImpPath + '/users.json';

		async.waterfall([
			// read users from file
			function (callback) {
				workManager.workStarted();
                winston.info('Importing users');
				fs.readJson(usersFile, { spaces: 4 }, function (err, users) {
					callback(err, users);
				});
			},
			// remove records currently present in db
			function (users, callback) {
				Users.find({}, function (err, usersDB) {
					var adminExists= false;//admin already present in db
					var adminsToRemove = [];
					var i = usersDB.length-1;
					while(i>=0){
						if(usersDB[i]._doc.local.username=="admin"){
							if(adminExists==false){
								adminExists=true;
							}
							else{
								adminsToRemove.push(usersDB[i]._id);
							}
						}
						i--;
					}
					var usersDBIds = Array.from(usersDB, function(x){ return x._id; });
					for(var i = 0; i<usersDBIds.length; i++){
						for(var j = 0; j<users.length;j++){
							if(usersDBIds[i]==users[j]._id || (adminExists==true && users[j].local.username=="admin")){ //filter out new admins as admin already exists
								users.splice(j,1);
							}
						}
					}
					//if duplicate admins in db, remove
					if(adminsToRemove.length){
						winston.info("Removing " +adminsToRemove.length+" duplicate admin from db");
						Users.deleteMany({_id: {$in :adminsToRemove}}, function(err, res){
							if(err){
								winston.error("Issue occured when deleting duplicate admin from db");
								callback(err);
							}
							else{
								callback(err, users);
							}
						});
					}else{
						callback(err, users);
					}
					
				});
			},
			// insert the users read from file
			function (users, callback) {
				// now insert
				Users.insertMany(users, function (err) {
					callback(err);
				});
			}
		],
			function (err, result) {
				// all done
                if (err) {
					winston.info('Import Users failed - Error - ', err);
				}

				if (Users != null) {
					Users.closeConnection();
				}

				workManager.workEnded();
			});

	};

	return Importer;
};